@@ -34,7 +34,7 @@ import math
 from urllib.request import urlretrieve
 from importlib import util
 
-program_version = "1.1.6"
+program_version = "1.1.7"
 eth_addr = "0x81F7B979fEa6134bA9FD5c701b3501A2e61E897a"
 btc_addr = "3CPmacGm34qYvR6XWLVEJmi2aNe3PZqUuq"
 
@@ -97,6 +97,7 @@ class KISS():
     CMD_STAT_SNR    = 0x24
     CMD_BLINK       = 0x30
     CMD_RANDOM      = 0x40
+    CMD_BOARD       = 0x47
     CMD_PLATFORM    = 0x48
     CMD_MCU         = 0x49
     CMD_FW_VERSION  = 0x50
@@ -137,6 +138,14 @@ class ROM():
     MODEL_A4       = 0xA4
     MODEL_A9       = 0xA9
 
+    PRODUCT_T32_20 = 0xB0
+    MODEL_B3       = 0xB3
+    MODEL_B8       = 0xB8
+
+    PRODUCT_T32_21 = 0xB1
+    MODEL_B4       = 0xB4
+    MODEL_B9       = 0xB9
+
     PRODUCT_TBEAM  = 0xE0
     MODEL_E4       = 0xE4
     MODEL_E9       = 0xE9
@@ -162,11 +171,21 @@ class ROM():
     INFO_LOCK_BYTE = 0x73
     CONF_OK_BYTE   = 0x73
 
+    BOARD_RNODE         = 0x31
+    BOARD_HMBRW         = 0x32
+    BOARD_TBEAM         = 0x33
+    BOARD_HUZZAH32      = 0x34
+    BOARD_GENERIC_ESP32 = 0x35
+    BOARD_LORA32_V2_0   = 0x36
+    BOARD_LORA32_V2_1   = 0x37
+
 mapped_product = ROM.PRODUCT_RNODE
 products = {
-    ROM.PRODUCT_RNODE: "RNode",
-    ROM.PRODUCT_HMBRW: "Hombrew RNode",
-    ROM.PRODUCT_TBEAM: "LilyGO T-Beam",
+    ROM.PRODUCT_RNODE:  "RNode",
+    ROM.PRODUCT_HMBRW:  "Hombrew RNode",
+    ROM.PRODUCT_TBEAM:  "LilyGO T-Beam",
+    ROM.PRODUCT_T32_20: "LilyGO LoRa32 v2.0",
+    ROM.PRODUCT_T32_21: "LilyGO LoRa32 v2.1",
 }
 
 platforms = {
@@ -183,6 +202,10 @@ mcus = {
 models = {
     0xA4: [410000000, 525000000, 14, "410 - 525 MHz", "rnode_firmware_latest.hex"],
     0xA9: [820000000, 1020000000, 17, "820 - 1020 MHz", "rnode_firmware_latest.hex"],
+    # 0xB3: [420000000, 520000000, 14, "420 - 520 MHz", "rnode_firmware_latest_lora32v20.zip"],
+    # 0xB8: [850000000, 950000000, 17, "850 - 950 MHz", "rnode_firmware_latest_lora32v20.zip"],
+    0xB4: [420000000, 520000000, 14, "420 - 520 MHz", "rnode_firmware_latest_lora32v21.zip"],
+    0xB9: [850000000, 950000000, 17, "850 - 950 MHz", "rnode_firmware_latest_lora32v21.zip"],
     0xE4: [420000000, 520000000, 14, "420 - 520 MHz", "rnode_firmware_latest_tbeam.zip"],
     0xE9: [850000000, 950000000, 17, "850 - 950 MHz", "rnode_firmware_latest_tbeam.zip"],
     0xFF: [100000000, 1100000000, 14, "(Band capabilities unknown)", None],
@@ -219,6 +242,7 @@ class RNode():
 
         self.provisioned = None
         self.product = None
+        self.board = None
         self.model = None
         self.hw_rev = None
         self.made = None
@@ -345,6 +369,9 @@ class RNode():
                                     self.minor_version = command_buffer[1]
                                     self.updateVersion()
 
+                        elif (command == KISS.CMD_BOARD):
+                            self.board = byte
+
                         elif (command == KISS.CMD_PLATFORM):
                             self.platform = byte
 
@@ -440,7 +467,7 @@ class RNode():
         self.version = str(self.major_version)+"."+minstr
 
     def detect(self):
-        kiss_command = bytes([KISS.FEND, KISS.CMD_DETECT, KISS.DETECT_REQ, KISS.FEND, KISS.CMD_FW_VERSION, 0x00, KISS.FEND, KISS.CMD_PLATFORM, 0x00, KISS.FEND, KISS.CMD_MCU, 0x00, KISS.FEND])
+        kiss_command = bytes([KISS.FEND, KISS.CMD_DETECT, KISS.DETECT_REQ, KISS.FEND, KISS.CMD_FW_VERSION, 0x00, KISS.FEND, KISS.CMD_PLATFORM, 0x00, KISS.FEND, KISS.CMD_MCU, 0x00, KISS.FEND, KISS.CMD_BOARD, 0x00, KISS.FEND])
         written = self.serial.write(kiss_command)
         if written != len(kiss_command):
             raise IOError("An IO error occurred while detecting hardware for "+self(str))
@@ -918,12 +945,14 @@ def main():
             print("[1] Original RNode")
             print("[2] Homebrew RNode")
             print("[3] LilyGO T-Beam")
+            print("[4] LilyGO LoRa32 v2.0")
+            print("[5] LilyGO LoRa32 v2.1")
             print("\n? ", end="")
 
             selected_product = None
             try:
                 c_dev = int(input())
-                if c_dev < 1 or c_dev > 3:
+                if c_dev < 1 or c_dev > 5:
                     raise ValueError()
                 elif c_dev == 1:
                     selected_product = ROM.PRODUCT_RNODE
@@ -949,6 +978,28 @@ def main():
                     print("please use an actual RNode from unsigned.io. Hit enter to continue.")
                     print("---------------------------------------------------------------------------")
                     input()
+                elif c_dev == 4:
+                    selected_product = ROM.PRODUCT_T32_20
+                    print("")
+                    print("---------------------------------------------------------------------------")
+                    print("Important! Using RNode firmware on LoRa32 devices should currently be")
+                    print("considered experimental. It is not intended for production or critical use.")
+                    print("The currently supplied firmware is provided AS-IS as a courtesey to those")
+                    print("who would like to experiment with it. If you want any degree of reliability,")
+                    print("please use an actual RNode from unsigned.io. Hit enter to continue.")
+                    print("---------------------------------------------------------------------------")
+                    input()
+                elif c_dev == 5:
+                    selected_product = ROM.PRODUCT_T32_21
+                    print("")
+                    print("---------------------------------------------------------------------------")
+                    print("Important! Using RNode firmware on LoRa32 devices should currently be")
+                    print("considered experimental. It is not intended for production or critical use.")
+                    print("The currently supplied firmware is provided AS-IS as a courtesey to those")
+                    print("who would like to experiment with it. If you want any degree of reliability,")
+                    print("please use an actual RNode from unsigned.io. Hit enter to continue.")
+                    print("---------------------------------------------------------------------------")
+                    input()
             except Exception as e:
                 print("That device type does not exist, exiting now.")
                 exit()
@@ -1025,6 +1076,50 @@ def main():
                     print("That band does not exist, exiting now.")
                     exit()
 
+            elif selected_product == ROM.PRODUCT_T32_20:
+                selected_mcu = ROM.MCU_ESP32
+                print("\nWhat band is this LoRa32 for?\n")
+                print("[1] 433 MHz")
+                print("[2] 868 MHz")
+                print("[3] 915 MHz")
+                print("[4] 923 MHz")
+                print("\n? ", end="")
+                try:
+                    c_model = int(input())
+                    if c_model < 1 or c_model > 4:
+                        raise ValueError()
+                    elif c_model == 1:
+                        selected_model = ROM.MODEL_B3
+                        selected_platform = ROM.PLATFORM_ESP32
+                    elif c_model > 1:
+                        selected_model = ROM.MODEL_B8
+                        selected_platform = ROM.PLATFORM_ESP32
+                except Exception as e:
+                    print("That band does not exist, exiting now.")
+                    exit()
+
+            elif selected_product == ROM.PRODUCT_T32_21:
+                selected_mcu = ROM.MCU_ESP32
+                print("\nWhat band is this LoRa32 for?\n")
+                print("[1] 433 MHz")
+                print("[2] 868 MHz")
+                print("[3] 915 MHz")
+                print("[4] 923 MHz")
+                print("\n? ", end="")
+                try:
+                    c_model = int(input())
+                    if c_model < 1 or c_model > 4:
+                        raise ValueError()
+                    elif c_model == 1:
+                        selected_model = ROM.MODEL_B4
+                        selected_platform = ROM.PLATFORM_ESP32
+                    elif c_model > 1:
+                        selected_model = ROM.MODEL_B9
+                        selected_platform = ROM.PLATFORM_ESP32
+                except Exception as e:
+                    print("That band does not exist, exiting now.")
+                    exit()
+
             if selected_model != ROM.MODEL_FF:
                 fw_filename = models[selected_model][4]
             else:
@@ -1034,8 +1129,6 @@ def main():
                     elif selected_mcu == ROM.MCU_2560:
                         fw_filename = "rnode_firmware_latest_m2560.hex"
                 elif selected_platform == ROM.PLATFORM_ESP32:
-                    # This variant is not released yet
-                    #fw_filename = "rnode_firmware_latest_esp32_generic.zip"
                     fw_filename = None
                     print("\nWhat kind of ESP32 board is this?\n")
                     print("[1] Adafruit Feather ESP32 (HUZZAH32)")
@@ -1225,6 +1318,23 @@ def main():
                             "0x10000", "./update/rnode_firmware_latest_tbeam.bin",
                             "0x8000", "./update/rnode_firmware_latest_tbeam.partitions",
                         ]
+                    elif fw_filename == "rnode_firmware_latest_lora32v21.zip":
+                        return [
+                            flasher,
+                            "--chip", "esp32",
+                            "--port", args.port,
+                            "--baud", "921600",
+                            "--before", "default_reset",
+                            "--after", "hard_reset",
+                            "write_flash", "-z",
+                            "--flash_mode", "dio",
+                            "--flash_freq", "80m",
+                            "--flash_size", "4MB",
+                            "0xe000", "./update/rnode_firmware_latest_lora32v21.boot_app0",
+                            "0x1000", "./update/rnode_firmware_latest_lora32v21.bootloader",
+                            "0x10000", "./update/rnode_firmware_latest_lora32v21.bin",
+                            "0x8000", "./update/rnode_firmware_latest_lora32v21.partitions",
+                        ]
                     elif fw_filename == "rnode_firmware_latest_featheresp32.zip":
                         return [
                             flasher,
@@ -1364,9 +1474,15 @@ def main():
                         elif rnode.mcu == ROM.MCU_2560:
                             fw_filename = "rnode_firmware_latest_m2560.hex"
                     elif rnode.platform == ROM.PLATFORM_ESP32:
-                        # This variant is not released yet
-                        #fw_filename = "rnode_firmware_latest_esp32.zip"
-                        fw_filename = None
+                        if rnode.board == ROM.BOARD_HUZZAH32:
+                            fw_filename = "rnode_firmware_latest_featheresp32.zip"
+                        elif rnode.board == ROM.BOARD_GENERIC_ESP32:
+                            fw_filename = "rnode_firmware_latest_esp32_generic.zip"
+                        else:
+                            fw_filename = None
+                            if args.update:
+                                RNS.log("ERROR: No firmware found for this board. Cannot update.")
+                                exit()
 
             if args.update:
                 rnode.disconnect()
@@ -1481,9 +1597,14 @@ def main():
                         else:
                             sigstring = "Genuine board, vendor is "+rnode.vendor
 
+                    if rnode.board != None:
+                        board_string = ":"+bytes([rnode.board]).hex()
+                    else:
+                        board_string = ""
+                        
                     RNS.log("")
                     RNS.log("Device info:")
-                    RNS.log("\tProduct            : "+products[rnode.product]+" "+models[rnode.model][3]+" ("+bytes([rnode.product]).hex()+":"+bytes([rnode.model]).hex()+")")
+                    RNS.log("\tProduct            : "+products[rnode.product]+" "+models[rnode.model][3]+" ("+bytes([rnode.product]).hex()+":"+bytes([rnode.model]).hex()+board_string+")")
                     RNS.log("\tDevice signature   : "+sigstring)
                     RNS.log("\tFirmware version   : "+rnode.version)
                     RNS.log("\tHardware revision  : "+str(int(rnode.hw_rev)))
